# -*- coding: utf-8 -*-
"""terrasnek Base Module

This module provides support for endpoints available in the Terraform Cloud
API. It's possible that some of these endpoints won't work if you're
working against an older version of Terraform Enterprise.

https://www.terraform.io/docs/cloud/api/index.html
"""

NAME = "terrasnek"
